%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Replication code for the paper "Sectoral Price Data and Models of Price Setting"                            %%%
%%%                                                                                                             %%%
%%% Copyright: Bartosz Mackowiak (Bartosz.Mackowiak@ecb.int), Emanuel Moench (emanuel.moench@ny.frb.org),       %%% 
%%% and Mirko Wiederholt (m-wiederholt@northwestern.edu)                                                        %%% 
%%%                                                                                                             %%%
%%% Implements a Gibbs sampler with Metropolis-Hastings Step to estimate the following factor model:            %%%
%%%                                                                                                             %%%
%%%     y_nt = beta_n(L)'*f_t + z_nt                                                                            %%%             
%%%     z_nt = phi_n(L)*z_{nt-1} + epsilon_nt                                                                   %%% 
%%%                                                                                                             %%% 
%%% The factors are white noise processes with identity variance covariance matrix.                             %%% 
%%% T is the sample size, N is the number of series in the cross-section,                                       %%% 
%%% k is the number of common factors,                                                                          %%% 
%%% beta is the N x k x(M+1) polynomial of factor loadings,                                                     %%% 
%%% phi is the N x(S+1) polynomial of autoregressive parameters of the z_n                                      %%% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% GENERAL SETTINGS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

dataset_filename = ['sublevel3_adj'];

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read in data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
% construct date series for plotting
start_year_dataset = 1985;
start_month_dataset = 1;
start_year_estimation = 1985;
start_month_estimation = 1;
end_year_estimation = 2005;
end_month_estimation = 5;

T1 = (start_year_estimation-start_year_dataset)*12 + start_month_estimation-start_month_dataset + 1;
T2 = (end_year_estimation-start_year_dataset)*12 + end_month_estimation - start_month_dataset + 1;
dates = start_year_estimation+(start_month_estimation/12):1./12:(end_year_estimation+end_month_estimation/12)';

% load dataset
[price_levels,var_names] = xlsread([dataset_filename,'.xls'],'Sheet1');
price_levels = price_levels';

% load group names
[data_mat_dum,name_mat] = xlsread([dataset_filename,'.xls'],'names','B3:C82');

% Compute monthly inflation rates
dates = dates(2:end);
infl_mly = diff(log(price_levels));
Y_all = infl_mly; 

% Remove outliers
Y_all_noout = zeros(size(Y_all));
for n = 1:size(Y_all,2),
    Y_all_noout(:,n) = remove_outliers(Y_all(:,n),'average');
end;        
Y_all = Y_all_noout;
Y = Y_all;
[T,N] = size(Y);
   
% Standardize data to have zero mean and unit variance 
Y = standardize(Y);

% store data
data.name_ser = name_mat(:,2)';
data.name_ser_group = name_mat(:,1)';
data.data = Y;

% setup calender
start_year  = start_year_dataset;
start_month = start_month_dataset+1; % due to differencing
end_year    = end_year_estimation;
end_month   = end_month_estimation; 
period = 1;
cal = [start_year,start_month,period];
i_year  = start_year;
i_month = start_month;
while ((i_year ~= end_year) || (i_month ~= end_month))
    if (i_month == 12)
        i_month = 1;
        i_year = i_year+1;
    else
        i_month = i_month+1;
    end
    period = period+1;
    cal = [ cal; [i_year,i_month,period] ];
end
data.dates = cal(:,1:2);

return
